#!/usr/bin/perl -w

use HTML::Tidy;

my $page = "";

my $field_headers = {
                     name => "Name",
                     owner => "Owner",
                     released => "Release Date",
                     origin => "Origin",
                     holder => "Holder",
                     retrieved => "<img src=\"http://www.geocaching.com/images/icons/picked_up.gif\" alt=\"retrieved\" />",
                     dropped => "<img src=\"http://www.geocaching.com/images/icons/dropped_off.gif\" alt=\"dropped\" />",
                     grabbed => "<img src=\"http://www.geocaching.com/images/icons/transfer.gif\" alt=\"grabbed\" />",
                     discovered => "<img src=\"http://www.geocaching.com/images/icons/icon_discovered.gif\" alt=\"discovered\" />",
                     note => "<img src=\"http://www.geocaching.com/images/icons/icon_note.gif\" alt=\"note\" />",
                     mileage => "Distance",
                     links => "\&nbsp;",
                     icon => "\&nbsp;",
                     reference => "TB #",
                     ranking => "\&nbsp;"
                    };
my $page_names = {
                  overview => "Overview",
                  ranking => "Ranking",
                  rules => "Rules",
                  index => "Home",
                  event => "Event",
                  blog => "Blog"
                 };

my $tidy = HTML::Tidy->new({
                            add_xml_decl => 1,
                            bare => 1,
                            clean => 1,
                            doctype => "strict",
                            drop_proprietary_attributes => 1,
                            numeric_entities => 1,
                            output_xhtml => 1,
                            quote_marks => 1,
                            indent => "auto",
                            output_encoding => "utf8",
                            tidy_mark => 1
                           });

sub trackable_sort_ascending_numeric {
 $trackables_data->{$a}->{$sortorder->{$page}[0][0]} <=> $trackables_data->{$b}->{$sortorder->{$page}[0][0]};
}

sub trackable_sort_descending_numeric {
 $trackables_data->{$b}->{$sortorder->{$page}[0][0]} <=> $trackables_data->{$a}->{$sortorder->{$page}[0][0]};
}

sub trackable_sort_ascending_alpha {
 $trackables_data->{$a}->{$sortorder->{$page}[0][0]} cmp $trackables_data->{$b}->{$sortorder->{$page}[0][0]};
}

sub trackable_sort_descending_alpha {
 $trackables_data->{$b}->{$sortorder->{$page}[0][0]} cmp $trackables_data->{$a}->{$sortorder->{$page}[0][0]};
}

sub trackable_sort {
 $funcname = "trackable_sort_".$sortorder->{$page}[0][1];
 &$funcname();
}

sub table_head {
 my $type = shift;
 my $fields = $fields->{$type};
 my $output = "";

 $output .= "<table><thead><tr>";

 for (@$fields) {
  $output .= "<td class=\"".$_."\">".$field_headers->{$_}."</td>";
 }

 $output .= "</tr></thead><tbody>";

 return $output;
}

sub table_row {
 my $tb = shift;
 my $ranking = shift;
 my $type = shift;
 my $fields = $fields->{$type};
 my $output = "";

 $output .= "<tr class=\"".($ranking % 2 == 0 ? "even" : "odd")."\">";

 for (@$fields) {
  if ($_ eq "ranking") {
   $output .= "<td class=\"ranking\">".$ranking."</td>";
  } elsif ($_ eq "mileage") {
   $output .= "<td class=\"mileage\">".sprintf("%.1f", ($unit eq "km" ? mi2km($tb->{mileage}) : $tb->{mileage}))." ".$unit."</td>";
  } else {
   $output .= "<td class=\"".$_."\">".$tb->{$_}."</td>";
  }
 }

 $output .= "</tr>";

 return $output;
}

sub table_footer {
 my $output = "</tbody></table>";
 return $output;
}

sub gen_menu {
 my $current = shift;
 my $output = "";

 foreach (@pages) {
  $output .= "<a href=\"".$_.".html\" ".($_ eq $current ? "class=\"current\"" : "").">".$page_names->{$_}."</a>";
 }

 foreach (@tables) {
  $output .= "<a href=\"".$_.".html\" ".($_ eq $current ? "class=\"current\"" : "").">".$page_names->{$_}."</a>";
 }

 foreach (keys(%$links)) {
  $output .= "<a href=\"".$links->{$_}."\">".$page_names->{$_}."</a>";
 }

 return $output;
}

sub output_html {
 my $page = shift;
 my $output = shift;

 my $outfile = $outdir."/".$page.".html";

 if (! open($fh_out, ">".$outfile)) {
  print "Could not open ".$outfile." for writing!\n";
  exit 1;
 }
 open($fh_in, "<files/template.html");
 
 my $buffer = "";

 while ($line = <$fh_in>) {
  my $menu = gen_menu($page);
  my $pagename = $page_names->{$page};

  $line =~ s/%%race%%/$race/gis;
  $line =~ s/%%page%%/$pagename/gis;
  $line =~ s/%%pages%%/$menu/gis;
  $line =~ s/%%content%%/$output/gis;
  $line =~ s/%%version%%/$VERSION/gis;
  $line =~ s/%%timestamp%%/$timestamp/gis;

  $buffer .= $line."\n";
 }

 print $fh_out $tidy->clean($buffer)."\n";

 close($fh_in);
 close($fh_out);
}

sub output_html_table {
 my $type = shift;
 
 my $output = table_head($type);
 my $ranking = 1;

 $page = $type;

 # Select correct sort method
 foreach (sort trackable_sort (keys(%$trackables_data))) {
  $output .= table_row($trackables_data->{$_}, $ranking++, $type);
 }
 $output .= table_footer();

 output_html($type, $output);
}

sub output_html_page {
 my $page = shift;
 my $output = ""; 

 open($page_in, "<pages/".$page.".html");
 while ($line = <$page_in>) {
  $output .= $line."\n";
 }
 close($page_in);

 output_html($page, $output);
}

1;
